@@ -9,16 +9,17 @@ https://gitlab.com/keatontaylor/alexapy
 """
 
 import asyncio
+import base64
+import binascii
+from binascii import Error
+import datetime
 from http.cookies import SimpleCookie
 from json import JSONDecodeError, dumps
-import base64
-import secrets
 import logging
-import datetime
 import os
 import pickle
 import re
-from binascii import Error
+import secrets
 from typing import Any, Callable, Dict, List, Optional, Text, Tuple, Union
 from urllib.parse import urlencode, urlparse
 
@@ -32,7 +33,7 @@ from yarl import URL
 from alexapy import aiohttp
 from alexapy.aiohttp.client_exceptions import ContentTypeError
 
-from .const import EXCEPTION_TEMPLATE, APP_NAME, USER_AGENT
+from .const import APP_NAME, EXCEPTION_TEMPLATE, LOCALE_KEY, USER_AGENT
 from .errors import AlexapyPyotpInvalidKey
 from .helpers import _catch_all_exceptions, delete_cookie, hide_serial, obfuscate
 
@@ -397,7 +398,8 @@ class AlexaLogin:
             )
             return False
         # Convert from amazon.com domain to native domain
-        if self._url != "amazon.com":
+        if self.url.lower() != "amazon.com":
+            self._headers["authority"] = f"www.{self._url}"
             await self.get_tokens()
             await self.exchange_token_for_cookies()
             get_resp = await self._session.get(
@@ -416,14 +418,23 @@ class AlexaLogin:
         self.customer_id = json.get("authentication", {}).get("customerId")
         if (email != "" and email.lower() == self._email.lower()) or email == "":
             if email != "":
-                _LOGGER.debug("Logged in as %s with id: %s", email, self.customer_id)
+                _LOGGER.debug(
+                    "Logged in as %s to %s with id: %s",
+                    email,
+                    self.url,
+                    self.customer_id,
+                )
             else:
                 _LOGGER.debug(
-                    "Logged in as mobile account %s with %s", email, self.customer_id
+                    "Logged in as to %s mobile account %s with %s",
+                    email,
+                    self.url,
+                    self.customer_id,
                 )
             self.stats["login_timestamp"] = datetime.datetime.now()
             self.stats["api_calls"] = 0
             await self.check_domain()
+            await self.save_cookiefile()
             return True
         _LOGGER.debug("Not logged in due to email mismatch")
         await self.reset()
@@ -441,7 +452,7 @@ class AlexaLogin:
                 "Accept-Language": "*",
                 "DNT": "1",
                 "Upgrade-Insecure-Requests": "1",
-                "authority": f"www.{self._url}",
+                "authority": "www.amazon.com",
             }
 
             #  initiate session
@@ -464,7 +475,7 @@ class AlexaLogin:
             )
         for cookie in cookie_jar:
             oldvalue = self._cookies[cookie.key] if cookie.key in self._cookies else ""
-            if cookie["domain"] == str(site) or cookie["domain"] == "":
+            if cookie["domain"] in [str(site), f".{site}", ""]:
                 self._cookies[cookie.key] = cookie.value
                 if self._debug and oldvalue != cookie.value:
                     _LOGGER.debug(
@@ -509,7 +520,7 @@ class AlexaLogin:
         _LOGGER.debug("Using credentials to log in")
         if not self._site:
             site: URL = URL("https://www.amazon.com/ap/signin")
-            deviceid: Text = "453535333833343639344134304445313832303944354342344141414342453123413249564c5635564d32573831"
+            deviceid: Text = f"{binascii.hexlify(secrets.token_hex(16).encode()).decode()}23413249564c5635564d32573831"
             query = {
                 "openid.return_to": "https://www.amazon.com/ap/maplanding",
                 "openid.assoc_handle": "amzn_dp_project_dee_ios",
@@ -525,7 +536,9 @@ class AlexaLogin:
                 "openid.ns": "http://specs.openid.net/auth/2.0",
                 "openid.pape.max_auth_age": "0",
                 "openid.oa2.scope": "device_auth_access",
-                "language": "en_US",
+                "language": LOCALE_KEY.get(self.url.replace("amazon", ""))
+                if LOCALE_KEY.get(self.url.replace("amazon", ""))
+                else "en_US",
             }
             site = site.update_query(query)
             _LOGGER.debug("Attempting oauth login to %s", site)
@@ -605,12 +618,12 @@ class AlexaLogin:
 
     async def save_cookiefile(self) -> None:
         """Save login session cookies to file."""
+        self._prepare_cookies_from_session(self.url)
         for cookiefile in self._cookiefile:
             if cookiefile == self._cookiefile[0]:
                 cookie_jar = self._session.cookie_jar
                 assert isinstance(cookie_jar, aiohttp.CookieJar)
-                cookie_jar.update_cookies(self._cookies, URL(self._url))
-                self._prepare_cookies_from_session(self._url)
+                cookie_jar.update_cookies(self._cookies, URL(self.url))
                 if self._debug:
                     _LOGGER.debug("Saving cookie to %s", cookiefile)
                 try:
@@ -646,51 +659,61 @@ class AlexaLogin:
         ).decode("utf8")
         self._cookies["frc"] = frc
         self._cookies["map-md"] = map_md
-        headers = {
-            "Content-Type": "application/json",
-            "Accept-Charset": "utf-8",
-            "x-amzn-identity-auth-domain": f"api.{self._url}",
-            "Connection": "keep-alive",
-            "Accept": "*/*",
-            "User-Agent": USER_AGENT,
-            "Accept-Language": "en-US",
-            # "Cookie": "; ".join(
-            #     [str(x) + "=" + str(y) for x, y in self._cookies.items()]
-            # ),
-        }
-        cookies = []
-        for k, value in self._cookies.items():
-            # if k == "csrf":
-            #     continue
-            cookies.append({"Value": value, "Name": k})
-        data = {
-            "requested_extensions": ["device_info", "customer_info"],
-            "cookies": {"website_cookies": cookies, "domain": f".{self._url}"},
-            "registration_data": {
-                "domain": "Device",
-                "app_version": "2.2.223830.0",
-                "device_type": "A2IVLV5VM2W81",
-                "device_name": f"%FIRST_NAME%'s%DUPE_STRATEGY_1ST%{APP_NAME}",
-                "os_version": "11.4.1",
-                "device_serial": "2cf947a5a093d686a30c33cb07a703fd"
-                if not self.uuid
-                else self.uuid,
-                "device_model": "iPhone",
-                "app_name": APP_NAME,
-                "software_version": "1",
-            },
-            "auth_data": {"access_token": self.access_token},
-            "user_context_map": {"frc": frc},
-            "requested_token_type": ["bearer", "mac_dms", "website_cookies"],
-        }
-
-        response = await self._session.post(
-            "https://api." + self._url + "/auth/register", json=data, headers=headers,
-        )
-        if response.status != 200:
+        if self.url.lower() != "amazon.com":
+            urls = [self.url, "amazon.com"]
+        else:
+            urls = [self.url]
+        registered = False
+        for url in urls:
+            headers = {
+                "Content-Type": "application/json",
+                "Accept-Charset": "utf-8",
+                "x-amzn-identity-auth-domain": f"api.{url}",
+                "Connection": "keep-alive",
+                "Accept": "*/*",
+                "User-Agent": USER_AGENT,
+                "Accept-Language": "en-US",
+                # "Cookie": "; ".join(
+                #     [str(x) + "=" + str(y) for x, y in self._cookies.items()]
+                # ),
+            }
+            cookies = []
+            for k, value in self._cookies.items():
+                # if k == "csrf":
+                #     continue
+                cookies.append({"Value": value, "Name": k})
+            data = {
+                "requested_extensions": ["device_info", "customer_info"],
+                "cookies": {"website_cookies": cookies, "domain": f".{url}"},
+                "registration_data": {
+                    "domain": "Device",
+                    "app_version": "2.2.223830.0",
+                    "device_type": "A2IVLV5VM2W81",
+                    "device_name": f"%FIRST_NAME%'s%DUPE_STRATEGY_1ST%{APP_NAME}",
+                    "os_version": "11.4.1",
+                    "device_serial": "2cf947a5a093d686a30c33cb07a703fd"
+                    if not self.uuid
+                    else self.uuid,
+                    "device_model": "iPhone",
+                    "app_name": APP_NAME,
+                    "software_version": "1",
+                },
+                "auth_data": {"access_token": self.access_token},
+                "user_context_map": {"frc": frc},
+                "requested_token_type": ["bearer", "mac_dms", "website_cookies"],
+            }
+            _LOGGER.debug("Attempting to register with %s", url)
+            response = await self._session.post(
+                "https://api." + url + "/auth/register", json=data, headers=headers,
+            )
+            # _LOGGER.debug("auth response %s with \n%s", response, dumps(data))
+            if response.status == 200:
+                registered = True
+                break
+        if not registered:
+            _LOGGER.debug("Unable to register with %s", urls)
             return False
         response = (await response.json()).get("response")
-        # _LOGGER.debug("auth response %s with \n%s", response, dumps(data))
         if response.get("success"):
             _LOGGER.debug(
                 "Successfully registered %s device with Amazon",
@@ -743,17 +766,17 @@ class AlexaLogin:
         headers = {
             "Content-Type": "application/x-www-form-urlencoded",
             "Accept-Charset": "utf-8",
-            "x-amzn-identity-auth-domain": f"api.{self._url}",
+            "x-amzn-identity-auth-domain": f"api.{self.url}",
             "Connection": "keep-alive",
             "Accept": "*/*",
             "User-Agent": USER_AGENT,
             "Accept-Language": "en-US",
-            "Cookie": "; ".join(
-                [str(x) + "=" + str(y) for x, y in self._cookies.items()]
-            ),
+            # "Cookie": "; ".join(
+            #     [str(x) + "=" + str(y) for x, y in self._cookies.items()]
+            # ),
         }
         response = await self._session.post(
-            "https://api." + self._url + "/auth/token", data=data, headers=headers,
+            "https://api." + self.url + "/auth/token", data=data, headers=headers,
         )
         if response.status != 200:
             if self._debug:
@@ -775,6 +798,8 @@ class AlexaLogin:
                 datetime.datetime.fromtimestamp(self.expires_in)
                 - datetime.datetime.now(),
             )
+            return True
+        return False
 
     async def exchange_token_for_cookies(self) -> bool:
         """Generate new session cookies using refresh token.
@@ -802,7 +827,7 @@ class AlexaLogin:
         headers = {
             "Content-Type": "application/x-www-form-urlencoded",
             "Accept-Charset": "utf-8",
-            "x-amzn-identity-auth-domain": f"api.{self._url}",
+            "x-amzn-identity-auth-domain": f"api.{self.url}",
             "Connection": "keep-alive",
             "Accept": "*/*",
             "User-Agent": USER_AGENT,
@@ -812,7 +837,7 @@ class AlexaLogin:
             # ),
         }
         response = await self._session.post(
-            "https://api." + self._url + "/ap/exchangetoken/cookies",
+            "https://api." + self.url + "/ap/exchangetoken/cookies",
             data=data,
             headers=headers,
         )
@@ -827,6 +852,7 @@ class AlexaLogin:
         response = (await response.json()).get("response")
         if self._debug:
             _LOGGER.debug("Exchange cookie json %s ", response)
+        success = False
         for domain, cookies in response["tokens"]["cookies"].items():
             # _LOGGER.debug("updating %s with %s", domain, cookies)
             for item in cookies:
@@ -843,15 +869,16 @@ class AlexaLogin:
                 for name, value in item.items():
                     if name in ["Name", "Value"]:
                         continue
-                    raw_cookie[cookie_name][name] = value
+                    raw_cookie[cookie_name][name] = f"{value}; Domain={domain}"
                 # _LOGGER.debug("updating jar with cookie %s", raw_cookie)
                 self._session.cookie_jar.update_cookies(raw_cookie, URL(domain))
-        for domain, cookies in response["tokens"]["cookies"].items():
             _LOGGER.debug(
                 "%s cookies successfully exchanged for refresh token for domain %s",
                 len(cookies),
                 domain,
             )
+            success = True
+        return success
 
     async def get_csrf(self) -> bool:
         """Generate csrf if missing.
@@ -863,7 +890,7 @@ class AlexaLogin:
         if self._cookies.get("csrf"):
             _LOGGER.debug("CSRF already exists; no need to discover")
             return True
-        _LOGGER.debug("Attemping to discover CSRF token")
+        _LOGGER.debug("Attempting to discover CSRF token")
         csrf_urls = [
             "/spa/index.html",
             "/api/language",
@@ -872,12 +899,12 @@ class AlexaLogin:
             "/api/strings",
         ]
         for url in csrf_urls:
-            response = await self._session.get(f"{self._prefix}{self._url}{url}")
+            response = await self._session.get(f"{self._prefix}{self.url}{url}")
             if response.status != 200:
                 if self._debug:
                     _LOGGER.debug("Unable to load page for csrf: %s", response)
                 continue
-            self._prepare_cookies_from_session(self._url)
+            self._prepare_cookies_from_session(self.url)
             if self._cookies.get("csrf"):
                 _LOGGER.debug("CSRF token found from %s", url)
                 return True
@@ -895,14 +922,14 @@ class AlexaLogin:
         headers = {
             "Content-Type": "application/x-www-form-urlencoded",
             "Accept-Charset": "utf-8",
-            "x-amzn-identity-auth-domain": f"api.{self._url}",
+            "x-amzn-identity-auth-domain": f"api.{self.url}",
             "Connection": "keep-alive",
             "Accept": "*/*",
             "User-Agent": USER_AGENT,
             "Accept-Language": "en-US",
         }
         response = await self._session.get(
-            f"{self._prefix}{self._url}/api/users/me?platform=ios&version=2.2.223830.0",
+            f"{self._prefix}{self.url}/api/users/me?platform=ios&version=2.2.223830.0",
             headers=headers,
         )
         if response.status != 200:
@@ -911,16 +938,14 @@ class AlexaLogin:
             return True
         response = await response.json()
         domain = response.get("marketPlaceDomainName")
-        if self._url not in domain:
+        if self.url not in domain:
             _LOGGER.warning(
                 "Domain %s does not match reported account domain %s; functionality is not likely to work, please fix",
                 self.url,
                 domain,
             )
             return False
-        _LOGGER.debug(
-            "Domain %s matches reported account domain: %s", self._url, domain
-        )
+        _LOGGER.debug("Domain %s matches reported account domain: %s", self.url, domain)
         return True
 
     async def _process_resp(self, resp) -> Text:
@@ -974,7 +999,7 @@ class AlexaLogin:
                     #               string,
                     #               href)
                     if href.startswith("/"):
-                        links[str(index)] = (string, (self._prefix + self._url + href))
+                        links[str(index)] = (string, (self._prefix + self.url + href))
                         index += 1
                     elif href.startswith("http"):
                         links[str(index)] = (string, href)
@@ -997,7 +1022,7 @@ class AlexaLogin:
                                 and item.get("type") == "hidden"
                             ):
                                 params[item.get("name")] = item.get("value")
-                        href = f"{self._prefix}{self._url}{action}?{urlencode(params)}"
+                        href = f"{self._prefix}{self.url}{action}?{urlencode(params)}"
                         links[str(index)] = (string, href)
                         index += 1
             if links:
@@ -1215,7 +1240,7 @@ class AlexaLogin:
                 # site = form_tag.find("input", {"name": "openid.return_to"}).get("value")
                 _LOGGER.debug("Found url for polling page %s", site)
             elif formsite and forgotpassword_tag:
-                site = self._prefix + self._url
+                site = self._prefix + self.url
                 _LOGGER.debug("Restarting login process %s", site)
             elif formsite:
                 site = formsite
